@@ -4,6 +4,7 @@ use strict;
 use warnings;
 
 use Coro;
+use Coro::AnyEvent;
 use Coro::Channel;
 use Coro::Handle;
 use Coro::ProcessPool::Util qw(encode decode $EOL);
@@ -13,29 +14,52 @@ if ($^O eq 'MSWin32') {
     die 'MSWin32 is not supported';
 }
 
+my $TIMEOUT = 0.2;
 my $IN  = unblock *STDIN;
 my $OUT = unblock *STDOUT;
 
 sub start {
     my $class   = shift;
+    my $inbox   = Coro::Channel->new();
     my $outbox  = Coro::Channel->new();
-    my $monitor = async {
+    my $running = 1;
+
+    my $in_worker = async {
+        while (1) {
+            Coro::AnyEvent::readable $IN->fh, $TIMEOUT or next;
+            my $line = $IN->readline($EOL) or last;
+            $inbox->put($line);
+        }
+
+        $running = 0;
+    };
+
+    my $out_worker = async {
         while (1) {
             my $line = $outbox->get or last;
             $OUT->print($line . $EOL);
         }
     };
 
-    scope_guard {
-        $outbox->shutdown;
-        $monitor->join;
+    my $proc_worker = async {
+        while (1) {
+            my $line  = $inbox->get or last;
+            my $data  = decode($line);
+            my $reply = $class->process_task($data);
+            $outbox->put(encode($reply));
+        }
     };
 
-    while (1) {
-        my $line  = $IN->readline($EOL) or last;
-        my $data  = decode($line);
-        my $reply = $class->process_task($data);
-        $outbox->put(encode($reply));
+    scope_guard {
+        $inbox->shutdown;
+        $in_worker->join;
+
+        $outbox->shutdown;
+        $out_worker->join;
+    };
+
+    while ($running) {
+        Coro::AnyEvent::sleep $TIMEOUT;
     }
 }
 
